//#############################################################################
//
// FILE:  ttplpfc_hal.c
//
// TITLE: Solution hardware abstraction layer
//        Low level functions and resources with hardware dependencies
//        Example: Functions to read current and voltage signals
//        Example: Functions to configure device peripherals
//
//#############################################################################
//#############################################################################
// $TI Release: TIDA_010062 v2.02.00.00 $
// $Release Date: Fri Oct  6 19:11:54 CDT 2023 $
// $Copyright:
// Copyright (C) 2023 Texas Instruments Incorporated - http://www.ti.com/
//
// ALL RIGHTS RESERVED
// $
//#############################################################################

#include <ttplpfc_hal.h>

/*  This routine sets up the basic device configuration such as initializing PLL
    copying code from FLASH to RAM */

// Transfer can be happen over single or double lane
FSI_DataWidth nLanes = FSI_DATA_WIDTH_1_LANE;

//
// Globals, these are not config parameters, user are not required to edit them
//
uint16_t *txBufAddr = 0, *rxBufAddr = 0;

void TTPLPFC_HAL_setupDevice(void)
{

    //
    // Initialize device clock and peripherals
    //
    Device_init();

    //
    // Disable pin locks and enable internal pull-ups.
    //
    Device_initGPIO();

    //
    // Initialize PIE and clear PIE registers. Disables CPU interrupts.
    //
    Interrupt_initModule();

    //
    // Initialize the PIE vector table with pointers to the shell Interrupt
    // Service Routines (ISR).
    //
    Interrupt_initVectorTable();

    //
    // initialize CPU timers
    // Initialize timer period to maximum
    //
    CPUTimer_setPeriod(CPUTIMER0_BASE, DEVICE_SYSCLK_FREQ / 100); // 100Hz-10ms
    CPUTimer_setPeriod(CPUTIMER1_BASE, DEVICE_SYSCLK_FREQ / 10); // 10Hz-100ms
    CPUTimer_setPeriod(CPUTIMER2_BASE, DEVICE_SYSCLK_FREQ / 10000);//10KHz-100us

    //
    // Initialize pre-scale counter to divide by 1 (SYSCLKOUT)
    //
    CPUTimer_setPreScaler(CPUTIMER0_BASE, 0);
    CPUTimer_setPreScaler(CPUTIMER1_BASE, 0);
    CPUTimer_setPreScaler(CPUTIMER2_BASE, 0);

    //
    // Make sure timer is stopped
    //
    CPUTimer_stopTimer(CPUTIMER0_BASE);
    CPUTimer_stopTimer(CPUTIMER1_BASE);
    CPUTimer_stopTimer(CPUTIMER2_BASE);
    CPUTimer_setEmulationMode(CPUTIMER0_BASE,
                              CPUTIMER_EMULATIONMODE_STOPAFTERNEXTDECREMENT);
    CPUTimer_setEmulationMode(CPUTIMER1_BASE,
                              CPUTIMER_EMULATIONMODE_STOPAFTERNEXTDECREMENT);
    CPUTimer_setEmulationMode(CPUTIMER2_BASE,
                              CPUTIMER_EMULATIONMODE_STOPAFTERNEXTDECREMENT);

    //
    // Reload all counter register with period value
    //
    CPUTimer_reloadTimerCounter(CPUTIMER0_BASE);
    CPUTimer_reloadTimerCounter(CPUTIMER1_BASE);
    CPUTimer_reloadTimerCounter(CPUTIMER2_BASE);

    CPUTimer_resumeTimer(CPUTIMER0_BASE);
    CPUTimer_resumeTimer(CPUTIMER1_BASE);
    CPUTimer_resumeTimer(CPUTIMER2_BASE);

}


void TTPLPFC_HAL_setupADC(void)
{
    //
    //Set ADC reference model and reference voltage.
    //
    ADC_setVREF(ADCA_BASE, ADC_REFERENCE_INTERNAL, ADC_REFERENCE_3_3V);

    ADC_setVREF(ADCB_BASE, ADC_REFERENCE_INTERNAL, ADC_REFERENCE_3_3V);
    ADC_setVREF(ADCC_BASE, ADC_REFERENCE_INTERNAL, ADC_REFERENCE_3_3V);




    //
    //Set ADCCLK divider to /2.
    //
    ADC_setPrescaler(ADCA_BASE, ADC_CLK_DIV_2_0);

    ADC_setPrescaler(ADCB_BASE, ADC_CLK_DIV_2_0);
    ADC_setPrescaler(ADCC_BASE, ADC_CLK_DIV_2_0);

    //
    //Power up the ADC.
    //
    ADC_enableConverter(ADCA_BASE);

    ADC_enableConverter(ADCB_BASE);
    ADC_enableConverter(ADCC_BASE);

    DEVICE_DELAY_US(1000);

    //
    //Setup conversion on ADC for PFC current and voltage.
    //Iin_sense:C6
    //
    ADC_setupSOC(TTPLPFC_IL_ADC_MODULE, TTPLPFC_IL_ADC_SOC_NO_1,
                 TTPLPFC_IL_ADC_TRIG_SOURCE_1, TTPLPFC_IL_ADC_PIN,
                 TTPLPFC_IL_ADC_ACQPS_SYS_CLKS);
    ADC_setupSOC(TTPLPFC_IL_ADC_MODULE, TTPLPFC_IL_ADC_SOC_NO_2,
                 TTPLPFC_IL_ADC_TRIG_SOURCE_2, TTPLPFC_IL_ADC_PIN,
                 TTPLPFC_IL_ADC_ACQPS_SYS_CLKS);

    //
    //AC_N:A3
    //
    ADC_setupSOC(TTPLPFC_VACN_ADC_MODULE, TTPLPFC_VACN_ADC_SOC_NO_1,
                 TTPLPFC_VACN_ADC_TRIG_SOURCE_1,
                 TTPLPFC_VACN_ADC_PIN, TTPLPFC_VACN_ADC_ACQPS_SYS_CLKS);
    ADC_setupSOC(TTPLPFC_VACN_ADC_MODULE, TTPLPFC_VACN_ADC_SOC_NO_2,
                 TTPLPFC_VACN_ADC_TRIG_SOURCE_2,
                 TTPLPFC_VACN_ADC_PIN, TTPLPFC_VACN_ADC_ACQPS_SYS_CLKS);

    //
    //AC_L:B3
    //
    ADC_setupSOC(TTPLPFC_VACL_ADC_MODULE, TTPLPFC_VACL_ADC_SOC_NO_1,
                 TTPLPFC_VACL_ADC_TRIG_SOURCE_1,
                 TTPLPFC_VACL_ADC_PIN, TTPLPFC_VACL_ADC_ACQPS_SYS_CLKS);
    ADC_setupSOC(TTPLPFC_VACL_ADC_MODULE, TTPLPFC_VACL_ADC_SOC_NO_2,
                 TTPLPFC_VACL_ADC_TRIG_SOURCE_2,
                 TTPLPFC_VACL_ADC_PIN, TTPLPFC_VACL_ADC_ACQPS_SYS_CLKS);

    //
    //AC_CUR_VREF:B0
    //
    ADC_setupSOC(TTPLPFC_IL_VREF_ADC_MODULE, TTPLPFC_IL_VREF_ADC_SOC_NO_1,
                 TTPLPFC_IL_VREF_ADC_TRIG_SOURCE_1, TTPLPFC_IL_VREF_ADC_PIN,
                 TTPLPFC_IL_VREF_ADC_ACQPS_SYS_CLKS);
    ADC_setupSOC(TTPLPFC_IL_VREF_ADC_MODULE, TTPLPFC_IL_VREF_ADC_SOC_NO_2,
                 TTPLPFC_IL_VREF_ADC_TRIG_SOURCE_2, TTPLPFC_IL_VREF_ADC_PIN,
                 TTPLPFC_IL_VREF_ADC_ACQPS_SYS_CLKS);
    //
    //VBUS:C8
    //
    ADC_setupSOC(TTPLPFC_VBUS_ADC_MODULE, TTPLPFC_VBUS_ADC_SOC_NO_1,
                 TTPLPFC_VBUS_ADC_TRIG_SOURCE_1,
                 TTPLPFC_VBUS_ADC_PIN, TTPLPFC_VBUS_ADC_ACQPS_SYS_CLKS);
    ADC_setupSOC(TTPLPFC_VBUS_ADC_MODULE, TTPLPFC_VBUS_ADC_SOC_NO_2,
                 TTPLPFC_VBUS_ADC_TRIG_SOURCE_2,
                 TTPLPFC_VBUS_ADC_PIN, TTPLPFC_VBUS_ADC_ACQPS_SYS_CLKS);
}


//
//Setup PWM module.
//
void TTPLPFC_HAL_setupPWM(uint32_t base1, uint32_t base3,
                          uint16_t pwm_period_ticks, uint16_t pwm_db_ticks_ls,
                          uint16_t pwm_db_ticks_hs)
{

    //
    // Time Base SubModule Registers
    //
    EPWM_setPeriodLoadMode(base1, EPWM_PERIOD_SHADOW_LOAD);
    EPWM_setTimeBasePeriod(base1, pwm_period_ticks >> 1);
    EPWM_setTimeBaseCounter(base1, 0);
    EPWM_setPhaseShift(base1, 0);
    EPWM_setTimeBaseCounterMode(base1, EPWM_COUNTER_MODE_UP_DOWN);
    EPWM_setClockPrescaler(base1, EPWM_CLOCK_DIVIDER_1, EPWM_HSCLOCK_DIVIDER_1);

    EPWM_setPeriodLoadMode(base3, EPWM_PERIOD_SHADOW_LOAD);
    EPWM_setTimeBasePeriod(base3, pwm_period_ticks >> 1);
    EPWM_setTimeBaseCounter(base3, 0);
    EPWM_setPhaseShift(base3, 0);
    EPWM_setTimeBaseCounterMode(base3, EPWM_COUNTER_MODE_UP_DOWN);
    EPWM_setClockPrescaler(base3, EPWM_CLOCK_DIVIDER_1, EPWM_HSCLOCK_DIVIDER_1);

    //
    // Counter Compare Submodule Registers
    // set duty 0% initially
    //
    EPWM_setCounterCompareValue(base1, EPWM_COUNTER_COMPARE_A, 0);
    EPWM_disableCounterCompareShadowLoadMode(base1, EPWM_COUNTER_COMPARE_A);

    EPWM_setCounterCompareValue(base3, EPWM_COUNTER_COMPARE_A, 0);
    EPWM_setCounterCompareShadowLoadMode(base3, EPWM_COUNTER_COMPARE_A,
                                         EPWM_COMP_LOAD_ON_CNTR_ZERO);

    //
    // Action Qualifier SubModule Registers
    // Action disable.
    //
    HWREGH(base1 + EPWM_O_AQCTLA) = 0 ;
    HWREGH(base3 + EPWM_O_AQCTLA) = 0 ;
    HWREGH(base1 + EPWM_O_AQCTLB) = 0 ;
    HWREGH(base3 + EPWM_O_AQCTLB) = 0 ;

    //
    // CTR = CMPA@UP , set EPWMxA/B to 0
    //
    EPWM_setActionQualifierAction(base3, EPWM_AQ_OUTPUT_A ,
            EPWM_AQ_OUTPUT_LOW, EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPA);
    EPWM_setActionQualifierAction(base3, EPWM_AQ_OUTPUT_B ,
            EPWM_AQ_OUTPUT_LOW, EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPA);

    //
    // CTR = CMPA@Down , set EPWMxA/B to 1
    //
    EPWM_setActionQualifierAction(base3, EPWM_AQ_OUTPUT_A ,
            EPWM_AQ_OUTPUT_HIGH, EPWM_AQ_OUTPUT_ON_TIMEBASE_DOWN_CMPA);
    EPWM_setActionQualifierAction(base3, EPWM_AQ_OUTPUT_B ,
            EPWM_AQ_OUTPUT_HIGH, EPWM_AQ_OUTPUT_ON_TIMEBASE_DOWN_CMPA);

    //
    //AQCSFRC Active Register Reload From Shadow Options
    //
    EPWM_setActionQualifierContSWForceShadowMode(base1,
            EPWM_AQ_SW_SH_LOAD_ON_CNTR_ZERO);
    EPWM_setActionQualifierContSWForceShadowMode(base3,
            EPWM_AQ_SW_SH_LOAD_ON_CNTR_ZERO);

    //
    //Dead-band generator Submodule Registers
    //set DBCTL Load Mode.Set the RED and FED shadow load mode
    //
    EPWM_setDeadBandControlShadowLoadMode(base3, EPWM_DB_LOAD_ON_CNTR_ZERO);
    EPWM_setRisingEdgeDelayCountShadowLoadMode(base3,
                                               EPWM_RED_LOAD_ON_CNTR_PERIOD);
    EPWM_setFallingEdgeDelayCountShadowLoadMode(base3,
                                                EPWM_FED_LOAD_ON_CNTR_ZERO);

    //
    // Active high complementary PWMs - Set up the deadband
    //
    EPWM_setDeadBandCounterClock(base3, EPWM_DB_COUNTER_CLOCK_FULL_CYCLE);
    EPWM_setRisingEdgeDelayCount(base3, pwm_db_ticks_hs);
    EPWM_setFallingEdgeDelayCount(base3, pwm_db_ticks_hs);
    EPWM_setDeadBandDelayMode(base3, EPWM_DB_RED, true);
    EPWM_setDeadBandDelayMode(base3, EPWM_DB_FED, true);
    EPWM_setRisingEdgeDeadBandDelayInput(base3, EPWM_DB_INPUT_EPWMA);
    EPWM_setFallingEdgeDeadBandDelayInput(base3, EPWM_DB_INPUT_EPWMA);
    EPWM_setDeadBandDelayPolarity(base3, EPWM_DB_FED,
                                  EPWM_DB_POLARITY_ACTIVE_LOW);
    EPWM_setDeadBandDelayPolarity(base3, EPWM_DB_RED,
                                  EPWM_DB_POLARITY_ACTIVE_HIGH);

    //
    // configure PWM 1 as master and Phase 2,3 and 4 as slaves and
    // let it pass the sync in pulse from PWM1
    //

    EPWM_disablePhaseShiftLoad(base1);
    EPWM_enableSyncOutPulseSource(base1, EPWM_SYNC_OUT_PULSE_ON_CNTR_ZERO);
    EPWM_disableOneShotSync(base1);

    EPWM_enablePhaseShiftLoad(base3);
    EPWM_setSyncInPulseSource(base3, EPWM_SYNC_IN_PULSE_SRC_SYNCOUT_EPWM1);
    EPWM_setPhaseShift(base3, 2);
    EPWM_setCountModeAfterSync(base3, EPWM_COUNT_MODE_UP_AFTER_SYNC);

}


void TTPLPFC_HAL_disablePWMCLKCounting(void)
{
    SysCtl_disablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);
}


void TTPLPFC_HAL_enablePWMCLKCounting(void)
{
    SysCtl_enablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);
}


void TTPLPFC_HAL_setupTrigForADC(uint32_t base)
{
    //
    // Select SOC from counter at ctr =CMPBD
    //
    EPWM_setADCTriggerSource(base, EPWM_SOC_A, EPWM_SOC_TBCTR_D_CMPB);

    //
    // set duty 0% initially
    //
    EPWM_setCounterCompareValue(base, EPWM_COUNTER_COMPARE_B, 25);

    //
    // Generate pulse on 1st even
    //
    EPWM_setADCTriggerEventPrescale(base, EPWM_SOC_A, 1);

    //
    // Enable SOC on A group
    //
    EPWM_enableADCTrigger(base, EPWM_SOC_A);

    //
    //SOCB Code
    //
    EPWM_setADCTriggerSource(base, EPWM_SOC_B, EPWM_SOC_TBCTR_D_CMPA );

    //
    // Generate pulse on 1st even
    //
    EPWM_setADCTriggerEventPrescale(base, EPWM_SOC_B, 1);

    //
    // Enable SOC on B group
    //
    EPWM_enableADCTrigger(base, EPWM_SOC_B);
}


void TTPLPFC_HAL_setupBoardProtection(uint32_t base1, uint32_t base3,
                                      float32_t current_limit,
                                      float32_t current_max_sense)
{
    //
    // Disable all the muxes first
    //
    XBAR_enableEPWMMux(XBAR_TRIP4, 0x00);

#if TTPLPFC_BOARD_PROTECTION_IL == 1

    //
    //Configure C6 as CMPSS3 HPMUX & LPMUX input pin in the analog sysctrl
    //
    ASysCtl_selectCMPHPMux(TTPLPFC_BOARD_PROTECTION_IL_CMPSS_ASYSCTRL_CMPHPMUX,
                         TTPLPFC_BOARD_PROTECTION_IL_CMPSS_ASYSCTRL_MUX_VALUE);
    ASysCtl_selectCMPLPMux(TTPLPFC_BOARD_PROTECTION_IL_CMPSS_ASYSCTRL_CMPLPMUX,
                         TTPLPFC_BOARD_PROTECTION_IL_CMPSS_ASYSCTRL_MUX_VALUE);

    //
    //Set up AC_CUR CMPSS
    //Set source for XBAR TRIP4 (TRIP4) from the CMPSS for AC_CUR (CTRIP3)
    //Enables the mux values to be passed to the X-BAR trip signal
    //
    TTPLPFC_HAL_setupCMPSS(TTPLPFC_BOARD_PROTECTION_IL_CMPSS_BASE,
                              current_limit, current_max_sense );

    XBAR_setEPWMMuxConfig(XBAR_TRIP4, TTPLPFC_BOARD_PROTECTION_IL_XBAR_MUX_VAL);
    XBAR_enableEPWMMux(XBAR_TRIP4, TTPLPFC_BOARD_PROTECTION_IL_XBAR_MUX);

#else
    #warning TTPLPFC_BOARD_PROTECTION_IL is disable
#endif

#if TTPLPFC_BOARD_PROTECTION_IL == 1

    //
    //Trip 4 is the input to the DCAHCOMPSEL. Output eventA1 when DCAH is set.
    //Set DC event source
    //
    EPWM_selectDigitalCompareTripInput(base1, EPWM_DC_TRIP_TRIPIN4,
                                       EPWM_DC_TYPE_DCAH);
    EPWM_setTripZoneDigitalCompareEventCondition(base1,
                                                 EPWM_TZ_DC_OUTPUT_A1,
                                                 EPWM_TZ_EVENT_DCXH_HIGH);
    EPWM_setDigitalCompareEventSource(base1, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1,
                                        EPWM_DC_EVENT_SOURCE_ORIG_SIGNAL);
    EPWM_setDigitalCompareEventSyncMode(base1, EPWM_DC_MODULE_A,
                                        EPWM_DC_EVENT_1,
                                        EPWM_DC_EVENT_INPUT_NOT_SYNCED);

    EPWM_selectDigitalCompareTripInput(base3,
                                       EPWM_DC_TRIP_TRIPIN4,
                                       EPWM_DC_TYPE_DCAH);
    EPWM_setTripZoneDigitalCompareEventCondition(base3,
                                                 EPWM_TZ_DC_OUTPUT_A1,
                                                 EPWM_TZ_EVENT_DCXH_HIGH);
    EPWM_setDigitalCompareEventSource(base3, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1,
                                        EPWM_DC_EVENT_SOURCE_ORIG_SIGNAL);
    EPWM_setDigitalCompareEventSyncMode(base3, EPWM_DC_MODULE_A,
                                        EPWM_DC_EVENT_1,
                                        EPWM_DC_EVENT_INPUT_NOT_SYNCED);

    //
    // Enable the following trips Emulator Stop,TZ1-3 and DCAEVT1
    //
    EPWM_enableTripZoneSignals(base1, EPWM_TZ_SIGNAL_DCAEVT1);
    EPWM_enableTripZoneSignals(base3, EPWM_TZ_SIGNAL_DCAEVT1);

#else
    #warning All current comparator based protections are disabled
#endif

    //
    //Enable the following trips Emulator Stop:TZ6 in CBC mode.
    //
    EPWM_enableTripZoneSignals(base1, EPWM_TZ_SIGNAL_CBC6);
    EPWM_enableTripZoneSignals(base3, EPWM_TZ_SIGNAL_CBC6);

    //
    // What do we want the OST/CBC events to do?
    // TZA events can force EPWMxA
    // TZB events can force EPWMxB
    //
    EPWM_setTripZoneAction(base1, EPWM_TZ_ACTION_EVENT_TZA, EPWM_TZ_ACTION_LOW);
    EPWM_setTripZoneAction(base1, EPWM_TZ_ACTION_EVENT_TZB, EPWM_TZ_ACTION_LOW);
    EPWM_setTripZoneAction(base3, EPWM_TZ_ACTION_EVENT_TZA, EPWM_TZ_ACTION_LOW);
    EPWM_setTripZoneAction(base3, EPWM_TZ_ACTION_EVENT_TZB, EPWM_TZ_ACTION_LOW);

    //
    //Clear any spurious trip and trip flag
    //
    EPWM_clearTripZoneFlag(base1,
                           (EPWM_TZ_INTERRUPT_OST | EPWM_TZ_INTERRUPT_DCAEVT1));
    EPWM_clearTripZoneFlag(base3,
                           (EPWM_TZ_INTERRUPT_OST | EPWM_TZ_INTERRUPT_DCAEVT1));

    //
    //Force TZ event
    //
    EPWM_forceTripZoneEvent(base1, EPWM_TZ_FORCE_EVENT_OST);
    EPWM_forceTripZoneEvent(base3, EPWM_TZ_FORCE_EVENT_OST);

/************************** End of Prot. Conf. ***************************/
}


void TTPLPFC_HAL_setupCMPSS(uint32_t base1,
                            float32_t current_limit,
                            float32_t current_max_sense )
{
    //
    //Enable CMPSS1
    //
    CMPSS_enableModule(base1);

    //
    //Use VDDA as the reference for comparator DACs
    //
    CMPSS_configDAC(base1,
        CMPSS_DACVAL_SYSCLK | CMPSS_DACREF_VDDA | CMPSS_DACSRC_SHDW);

    //
    //Set DAC to H~90% and L ~10% valuess
    //
    CMPSS_setDACValueHigh(base1,
                          2048 +
                          (int16_t)((float32_t)current_limit *
                                  (float32_t)2048.0 /
                                  (float32_t)current_max_sense));
    CMPSS_setDACValueLow(base1,
                         2048 -
                         (int16_t)((float32_t)current_limit *
                                 (float32_t)2048.0 /
                                 (float32_t)current_max_sense));

    //
    //High Comparator output is "not" inverted for high compare event
    //Low Comparator output is inverted for for low compare event
    //
    CMPSS_configHighComparator(base1, CMPSS_INSRC_DAC );
    CMPSS_configLowComparator(base1, CMPSS_INSRC_DAC | CMPSS_INV_INVERTED);

    //
    //CMPSS_configFilterHigh(base, samplePrescale,sampleWindow,threshold);
    //Every 2 sysclk sample once, input 10 samples,
    //when >= 7 samples are same, output this value
    //
    CMPSS_configFilterHigh(base1, 2, 10, 7);
    CMPSS_configFilterLow(base1, 2, 10, 7);

    //
    //Reset filter logic & start filtering
    //
    CMPSS_initFilterHigh(base1);
    CMPSS_initFilterLow(base1);

    //
    //Configure CTRIPOUT path: Digital filter connect CTRIPOUT3
    //
    CMPSS_configOutputsHigh(base1, CMPSS_TRIP_FILTER | CMPSS_TRIP_FILTER);
    CMPSS_configOutputsLow(base1, CMPSS_TRIP_FILTER | CMPSS_TRIP_FILTER);

    //
    //Comparator hysteresis control , set to 2x typical value
    //
    CMPSS_setHysteresis(base1, 2);

    //
    //Clear the latched comparator events
    //
    CMPSS_clearFilterLatchHigh(base1);
    CMPSS_clearFilterLatchLow(base1);
}


void TTPLPFC_HAL_setupLEDGPIO(void)
{
    GPIO_setDirectionMode(TTPLPFC_GPIO_LED1, GPIO_DIR_MODE_OUT);
    GPIO_setDirectionMode(TTPLPFC_GPIO_LED2, GPIO_DIR_MODE_OUT);
    GPIO_setQualificationMode(TTPLPFC_GPIO_LED1, GPIO_QUAL_SYNC);
    GPIO_setQualificationMode(TTPLPFC_GPIO_LED2, GPIO_QUAL_SYNC);
    GPIO_setPinConfig(TTPLPFC_GPIO_LED1_PIN_CONFIG);
    GPIO_setPinConfig(TTPLPFC_GPIO_LED2_PIN_CONFIG);
}


void TTPLPFC_HAL_toggleLED(void)
{
    static uint16_t ledCnt1 = 0;

    if(ledCnt1 == 0)
    {
        GPIO_togglePin(TTPLPFC_GPIO_LED1);
        ledCnt1 = 5;
    }
    else
    {
        ledCnt1--;
    }

}


void TTPLPFC_HAL_setupProfilingGPIO(void)
{
    GPIO_setDirectionMode(TTPLPFC_GPIO_PROFILING1, GPIO_DIR_MODE_OUT);
    GPIO_setDirectionMode(TTPLPFC_GPIO_PROFILING2, GPIO_DIR_MODE_OUT);
    GPIO_setQualificationMode(TTPLPFC_GPIO_PROFILING1, GPIO_QUAL_SYNC);
    GPIO_setQualificationMode(TTPLPFC_GPIO_PROFILING2, GPIO_QUAL_SYNC);
    GPIO_setPinConfig(TTPLPFC_GPIO_PROFILING1_PIN_CONFIG);
    GPIO_setPinConfig(TTPLPFC_GPIO_PROFILING2_PIN_CONFIG);

#if TTPLPFC_CONTROL_RUNNING_ON == CLA_CORE
    GPIO_setMasterCore(TTPLPFC_GPIO_PROFILING1, GPIO_CORE_CPU1_CLA1);
    GPIO_setMasterCore(TTPLPFC_GPIO_PROFILING2, GPIO_CORE_CPU1_CLA1);
#endif

}


void TTPLPFC_HAL_setupRelayGPIO(void)
{
    GPIO_setDirectionMode(TTPLPFC_GPIO_RELAY, GPIO_DIR_MODE_OUT);
    GPIO_setQualificationMode(TTPLPFC_GPIO_RELAY, GPIO_QUAL_SYNC);
    GPIO_setPinConfig(TTPLPFC_GPIO_RELAY_PIN_CONFIG);

#if TTPLPFC_CONTROL_RUNNING_ON == CLA_CORE
    GPIO_setMasterCore(TTPLPFC_GPIO_RELAY, GPIO_CORE_CPU1_CLA1);
#endif

}


void TTPLPFC_HAL_setupGainChangeGPIO(void)
{
    GPIO_setDirectionMode(TTPLPFC_GPIO_I_SENSE_GAIN_ADJUST, GPIO_DIR_MODE_OUT);
    GPIO_setQualificationMode(TTPLPFC_GPIO_I_SENSE_GAIN_ADJUST, GPIO_QUAL_SYNC);
    GPIO_setPinConfig(TTPLPFC_GPIO_I_SENSE_GAIN_ADJUST_PIN_CONFIG);
}

void TTPLPFC_HAL_setPinsAsPWM()
{
    GPIO_writePin(TTPLPFC_LOW_FREQ_PWM_H_GPIO, 0);
    GPIO_writePin(TTPLPFC_LOW_FREQ_PWM_L_GPIO, 0);

    GPIO_writePin(TTPLPFC_HIGH_FREQ_PWM_H_GPIO, 0);
    GPIO_writePin(TTPLPFC_HIGH_FREQ_PWM_L_GPIO, 0);

    GPIO_setDirectionMode( TTPLPFC_LOW_FREQ_PWM_H_GPIO, GPIO_DIR_MODE_OUT);
    GPIO_setPadConfig(TTPLPFC_LOW_FREQ_PWM_H_GPIO, GPIO_PIN_TYPE_STD);
    GPIO_setPinConfig(TTPLPFC_LOW_FREQ_PWM_H_GPIO_PIN_CONFIG);

    GPIO_setDirectionMode(TTPLPFC_LOW_FREQ_PWM_L_GPIO, GPIO_DIR_MODE_OUT);
    GPIO_setPadConfig(TTPLPFC_LOW_FREQ_PWM_L_GPIO, GPIO_PIN_TYPE_STD);
    GPIO_setPinConfig(TTPLPFC_LOW_FREQ_PWM_L_GPIO_PIN_CONFIG);

    GPIO_setDirectionMode(TTPLPFC_HIGH_FREQ_PWM_H_GPIO, GPIO_DIR_MODE_OUT);
    GPIO_setPadConfig(TTPLPFC_HIGH_FREQ_PWM_H_GPIO, GPIO_PIN_TYPE_STD);
    GPIO_setPinConfig(TTPLPFC_HIGH_FREQ_PWM_H_GPIO_PIN_CONFIG);

    GPIO_setDirectionMode(TTPLPFC_HIGH_FREQ_PWM_L_GPIO, GPIO_DIR_MODE_OUT);
    GPIO_setPadConfig(TTPLPFC_HIGH_FREQ_PWM_L_GPIO, GPIO_PIN_TYPE_STD);
    GPIO_setPinConfig(TTPLPFC_HIGH_FREQ_PWM_L_GPIO_PIN_CONFIG);

}

//
// initFSI - Initializes FSI Tx/Rx and also sends FLUSH sequence.
//
void TTPLPFC_HAL_initFSI(void)
{
    FSI_disableRxInternalLoopback(FSIRXA_BASE);

    //
    // NOTE: External loopback, Modify GPIO settings as per setup
    //
    GPIO_setPinConfig(DEVICE_GPIO_CFG_FSI_TXCLK);
    GPIO_setPinConfig(DEVICE_GPIO_CFG_FSI_TX0);

    GPIO_setPinConfig(DEVICE_GPIO_CFG_FSI_RXCLK);
    GPIO_setPinConfig(DEVICE_GPIO_CFG_FSI_RX0);

    if(nLanes == FSI_DATA_WIDTH_2_LANE)
    {
        GPIO_setPinConfig(DEVICE_GPIO_CFG_FSI_TX1);
        GPIO_setPinConfig(DEVICE_GPIO_CFG_FSI_RX1);
    }

    //
    // Set RX GPIO to be asynchronous
    // (pass through without delay)
    // Default setting is to have 2 SYS_CLK cycles delay
    //
    if(nLanes == FSI_DATA_WIDTH_2_LANE)
    {
        GPIO_setQualificationMode(DEVICE_GPIO_PIN_FSI_RX1, GPIO_QUAL_ASYNC);
    }
    GPIO_setQualificationMode(DEVICE_GPIO_PIN_FSI_RX0, GPIO_QUAL_ASYNC);
    GPIO_setQualificationMode(DEVICE_GPIO_PIN_FSI_RXCLK, GPIO_QUAL_ASYNC);

    // Could add logic to calculate PRESCALER_VAL based on user input FSI CLK
    FSI_performTxInitialization(FSITXA_BASE, TTPLPFC_PRESCALER_VAL);
    FSI_performRxInitialization(FSIRXA_BASE);
    txBufAddr = (uint16_t *)FSI_getTxBufferAddress(FSITXA_BASE);
    rxBufAddr = (uint16_t *)FSI_getRxBufferAddress(FSIRXA_BASE);
}


void TTPLPFC_HAL_setupCLA(void)
{
    //
    // setup CLA to register an interrupt
    //
#if TTPLPFC_CONTROL_RUNNING_ON == CLA_CORE

    //
    //move CLA code from flash to shared RAM by C28x.configured in .cmd
    //
    memcpy((uint32_t *)&Cla1ProgRunStart, (uint32_t *)&Cla1ProgLoadStart,
            (uint32_t)&Cla1ProgLoadSize );

    //
    // first assign memory to CLA
    //
    MemCfg_setLSRAMMasterSel(MEMCFG_SECT_LS0, MEMCFG_LSRAMMASTER_CPU_CLA1);
    MemCfg_setLSRAMMasterSel(MEMCFG_SECT_LS1, MEMCFG_LSRAMMASTER_CPU_CLA1);
    MemCfg_setLSRAMMasterSel(MEMCFG_SECT_LS2, MEMCFG_LSRAMMASTER_CPU_CLA1);
    MemCfg_setLSRAMMasterSel(MEMCFG_SECT_LS3, MEMCFG_LSRAMMASTER_CPU_CLA1);
    MemCfg_setLSRAMMasterSel(MEMCFG_SECT_LS4, MEMCFG_LSRAMMASTER_CPU_CLA1);
    MemCfg_setLSRAMMasterSel(MEMCFG_SECT_LS5, MEMCFG_LSRAMMASTER_CPU_CLA1);

    MemCfg_setCLAMemType(MEMCFG_SECT_LS0, MEMCFG_CLA_MEM_DATA);
    MemCfg_setCLAMemType(MEMCFG_SECT_LS1, MEMCFG_CLA_MEM_DATA);
    MemCfg_setCLAMemType(MEMCFG_SECT_LS2, MEMCFG_CLA_MEM_PROGRAM);
    MemCfg_setCLAMemType(MEMCFG_SECT_LS3, MEMCFG_CLA_MEM_PROGRAM);
    MemCfg_setCLAMemType(MEMCFG_SECT_LS4, MEMCFG_CLA_MEM_PROGRAM);
    MemCfg_setCLAMemType(MEMCFG_SECT_LS5, MEMCFG_CLA_MEM_PROGRAM);

    //
    // Suppressing #770-D conversion from pointer to smaller integer
    // The CLA address range is 16 bits so the addresses passed to the MVECT
    // registers will be in the lower 64KW address space. Turn the warning
    // back on after the MVECTs are assigned addresses
    //
    #pragma diag_suppress = 770

    CLA_mapTaskVector(CLA1_BASE, CLA_MVECT_1, (uint16_t)&Cla1Task1);
    CLA_mapTaskVector(CLA1_BASE, CLA_MVECT_2, (uint16_t)&Cla1Task2);
    CLA_mapTaskVector(CLA1_BASE, CLA_MVECT_3, (uint16_t)&Cla1Task3);
    CLA_mapTaskVector(CLA1_BASE, CLA_MVECT_4, (uint16_t)&Cla1Task4);
    CLA_mapTaskVector(CLA1_BASE, CLA_MVECT_5, (uint16_t)&Cla1Task5);
    CLA_mapTaskVector(CLA1_BASE, CLA_MVECT_6, (uint16_t)&Cla1Task6);
    CLA_mapTaskVector(CLA1_BASE, CLA_MVECT_7, (uint16_t)&Cla1Task7);
    CLA_mapBackgroundTaskVector(CLA1_BASE, (uint16_t)&Cla1BackgroundTask);

    #pragma diag_warning = 770

    CLA_enableIACK(CLA1_BASE);
    CLA_enableTasks(CLA1_BASE, CLA_TASKFLAG_ALL);

    //
    //Used to debug, the below line is intentionally commented
    //CLA_forceTasks(CLA1_BASE, CLA_TASKFLAG_1);
    //

    CLA_enableHardwareTrigger(CLA1_BASE);
    CLA_setTriggerSource(CLA_TASK_8, TTPLPFC_CLA_INSTRUMENTATIONISR_TRIG);
    CLA_enableBackgroundTask(CLA1_BASE);

    CLA_setTriggerSource(CLA_TASK_1, TTPLPFC_CLA_CONTROLISR_TRIG);

    TTPLPFC_HAL_enablePWMInterruptGeneration(EPWM1_BASE);
    TTPLPFC_HAL_clearPWMInterruptFlag(
            TTPLPFC_C28x_CONTROLISR_INTERRUPT_TRIG_PWM_BASE);


#endif
}

