//#############################################################################
//
// FILE:   ttplpfc_main.c
//
// TITLE:  This is the main file for the solution
//         Additional solution support files are
//             <solution>.c -> solution source file
//             <solution>.h -> solution header file
//             <solution>_settings.h -> powerSUITE generated settings
//             <solution>_hal.c -> device drivers source file
//             <solution>_hal.h -> device drivers header file
//
//#############################################################################
//#############################################################################
// $TI Release: TIDA_010062 v2.02.00.00 $
// $Release Date: Fri Oct  6 19:11:54 CDT 2023 $
// $Copyright:
// Copyright (C) 2023 Texas Instruments Incorporated - http://www.ti.com/
//
// ALL RIGHTS RESERVED
// $
//#############################################################################


//
// the includes
//

#include <ttplpfc.h>

//
//*********************** C2000 SYSTEM SW FRAMEWORK  ************************//
//

//
//---  State Machine Related ---
//
int16_t vTimer0[4];         // Virtual Timers slaved off CPU Timer 0 (A events)
int16_t vTimer1[4];         // Virtual Timers slaved off CPU Timer 1 (B events)

//
// Variable declarations for state machine
//
void (*Alpha_State_Ptr)(void);  // Base States pointer
void (*A_Task_Ptr)(void);       // State pointer A branch
void (*B_Task_Ptr)(void);       // State pointer B branch

//
// State Machine function prototypes
//
void A0(void);  //state A0
void B0(void);  //state B0
//
// A branch states
//
void A1(void);  //state A1
//
// B branch states
//
void B1(void);  //state B1
void B2(void);  //state B2
void B3(void);  //state B3


void main(void)
{
    //
    // This routine sets up the basic device configuration such as
    // initializing PLL, copying code from FLASH to RAM,
    // this routine will also initialize the CPU timers that are used in
    // the background task for this system
    //
    TTPLPFC_HAL_setupDevice();

    //
    // Tasks State-machine init
    //
    Alpha_State_Ptr = &A0;
    A_Task_Ptr = &A1;
    B_Task_Ptr = &B1;

    //
    // Stop all PWM mode clock
    //
    TTPLPFC_HAL_disablePWMCLKCounting();

    //
    //Sets up PWM for 1PH TTPL PFC. PWM1 is for low side, PWM3 is for high side
    //
    TTPLPFC_HAL_setupPWM(TTPLPFC_LOW_FREQ_PWM_BASE,
                         TTPLPFC_HIGH_FREQ_PWM_BASE,
                         TTPLPFC_PFC_PWM_PERIOD,
                         TTPLPFC_HIGH_FREQ_PWM_DEADBAND_RED_COUNT,
                         TTPLPFC_HIGH_FREQ_PWM_DEADBAND_FED_COUNT);

    //
    // power up ADC on the device
    //
    TTPLPFC_HAL_setupADC();

    //
    //Configure profiling GPIO
    //
    TTPLPFC_HAL_setupProfilingGPIO();

    //
    //Configure LED GPIO
    //
    TTPLPFC_HAL_setupLEDGPIO();

    //
    //Configure relay GPIO
    //
    TTPLPFC_HAL_setupRelayGPIO();

    //
    //Configure gain change GPIO and current sample gain
    //
    TTPLPFC_HAL_setupGainChangeGPIO();
    TTPLPFC_HAL_gainHighSetforLowCurrent();

    //
    // Initialize global variables generic to the board like ones used to
    // read current values.
    //
    TTPLPFC_globalVariablesInit();

    //
    //Setup SFRA
    //
    #if TTPLPFC_SFRA_TYPE == TTPLPFC_SFRA_DISABLED
    #else
        TTPLPFC_setupSFRA();
    #endif

    //
    // Enable PWM Clocks
    //
    TTPLPFC_HAL_enablePWMCLKCounting();

    //
    //Set up PWM trigger ADC conversion.
    //
    TTPLPFC_HAL_setupTrigForADC(TTPLPFC_HIGH_FREQ_PWM_BASE);

    //
    //Offset Calibration Routine  TTPLPFC_iLMeasOffset_pu=498000056;
    //
    TTPLPFC_calibrateOffset(&TTPLPFC_iLMeasOffset_pu,
                            TTPLPFC_k1, TTPLPFC_k2);

    //
    // note the value below is determined by looking at the DC component
    //
    TTPLPFC_iLMeasOffset_pu = 0.498f;

    //
    // setup protection and trips for the board
    //
    TTPLPFC_HAL_setupBoardProtection(TTPLPFC_LOW_FREQ_PWM_BASE,
                                     TTPLPFC_HIGH_FREQ_PWM_BASE,
                                     TTPLPFC_IL_TRIP_LIMIT_AMPS,
                                     TTPLPFC_IL_MAX_SENSE_AMPS);

    //
    //safe to setup PWM pins
    //
    TTPLPFC_HAL_setPinsAsPWM();

    //
    //ISR Mapping
    //
    TTPLPFC_HAL_setupInterrupt();

    //
    // Initialize basic settings for FSI
    //
    TTPLPFC_HAL_initFSI();


#if TTPLPFC_FSI_ISR
    TTPLPFC_HAL_setupFsiInterrupt();
#else

#endif

    //
    // Send the flush sequence
    //
    TTPLPFC_HAL_FsiSendFlushSequence();

    //
    // IDLE loop. Just sit and loop forever, periodically will branch into
    // A0-A3, B0-B3, C0-C3 tasks
    // Frequency of this branching is set in setupDevice routine:
    //
    for(;;)
    {
        //
        // State machine entry & exit point
        //
        (*Alpha_State_Ptr)();   // jump to an Alpha state (A0,B0,...)

        if(TTPLPFC_initializationFlag == 1)
        {
            TTPLPFC_initializationFlag = 0;
            TTPLPFC_sfra1.start = 1;
        }
    }
} //END MAIN CODE



#if TTPLPFC_CONTROL_RUNNING_ON == C28x_CORE
interrupt void ISR1(void)
{
    TTPLPFC_HAL_setProfilingGPIO();

    #if TTPLPFC_LAB == 1
        TTPLPFC_runISR1_lab1();
    #elif TTPLPFC_LAB == 2
        TTPLPFC_runISR1_lab2();
    #elif TTPLPFC_LAB == 3
        TTPLPFC_runISR1_lab3();
    #elif TTPLPFC_LAB == 4
        TTPLPFC_runISR1_lab4();
    #else
        #warning undefined lab
    #endif

    TTPLPFC_HAL_clearInterrupt(
            TTPLPFC_C28x_CONTROLISR_INTERRUPT_PIE_GROUP_NO);

    TTPLPFC_HAL_resetProfilingGPIO();
}
#endif


#if TTPLPFC_INSTRUMENTATION_ISR_RUNNING_ON == C28x_CORE
interrupt void ISR2(void)
{
    TTPLPFC_HAL_setProfilingGPIO2();

    TTPLPFC_runISR2();

    TTPLPFC_HAL_resetProfilingGPIO2();
}
#endif

//
// fsiTxInt1ISR - FSI Tx Interrupt on INsT1 line
//
__interrupt void fsiTxInt1ISR(void)
{
    TTPLPFC_runFsiTxISR1();

    TTPLPFC_HAL_clearInterrupt(INTERRUPT_ACK_GROUP7);
}

//
// fsiTxInt2ISR - FSI Tx Interrupt on INT2 line
//
__interrupt void fsiTxInt2ISR(void)
{
    TTPLPFC_runFsiTxISR2();

    TTPLPFC_HAL_clearInterrupt(INTERRUPT_ACK_GROUP7);

    TTPLPFC_HAL_disableAllFSIInterrupts();
}

//
// fsiRxInt1ISR - FSI Rx Interrupt on INT1 line
//
__interrupt void fsiRxInt1ISR(void)
{
    TTPLPFC_runFsiRxISR1();

    TTPLPFC_HAL_clearInterrupt(INTERRUPT_ACK_GROUP7);
}

//
// fsiRxInt2ISR - FSI Rx Interrupt on INT2 line
//
__interrupt void fsiRxInt2ISR(void)
{
    TTPLPFC_runFsiRxISR2();

    TTPLPFC_HAL_clearInterrupt(INTERRUPT_ACK_GROUP7);

    TTPLPFC_HAL_disableAllFSIInterrupts();
}

//
// STATE-MACHINE SEQUENCING AND SYNCRONIZATION FOR SLOW BACKGROUND TASKS
//

//
//--------------------------------- FRAME WORK --------------------------------
//
void A0(void)
{
    //
    // loop rate synchronizer for A-tasks
    //
    if(TTPLPFC_GET_TASK_A_TIMER_OVERFLOW_STATUS == 1)
    {
        TTPLPFC_CLEAR_TASK_A_TIMER_OVERFLOW_FLAG;   // clear flag

        (*A_Task_Ptr)();    // jump to an A Task (A1,A2,A3,...)

        vTimer0[0]++;       // virtual timer 0, instance 0 (spare)
    }

    //
    // TTPLPFC_relayControl();
    // Comment out to allow only A tasks
    //
    Alpha_State_Ptr = &B0;
}

void B0(void)
{
    //
    // loop rate synchronizer for B-tasks
    //
    if(TTPLPFC_GET_TASK_B_TIMER_OVERFLOW_STATUS  == 1)
    {
        TTPLPFC_CLEAR_TASK_B_TIMER_OVERFLOW_FLAG;   // clear flag

        (*B_Task_Ptr)();    // jump to a B Task (B1,B2,B3,...)

        vTimer1[0]++;       // virtual timer 1, instance 0 (spare)
    }

    //
    // Allow A state tasks
    //
    Alpha_State_Ptr = &A0;
}

//
// A - TASKS (executed in every 10 msec)
//
void A1(void)
{
#if TTPLPFC_SFRA_TYPE != TTPLPFC_SFRA_DISABLED
    TTPLPFC_runSFRABackGroundTasks();
#endif

    //
    //the next time CpuTimer0 'counter' reaches Period value go to A2
    //
    A_Task_Ptr = &A1;
}

//
// B - TASKS (executed in every 100 msec)
//
void B1(void)
{
    TTPLPFC_updateBoardStatus();
    TTPLPFC_setLabIndicatorVariable();

    //
    //the next time CpuTimer1 'counter' reaches Period value go to B2
    //
    B_Task_Ptr = &B2;
}

void B2(void) //  SPARE
{

    TTPLPFC_autoStart();

    //
    //the next time CpuTimer1 'counter' reaches Period value go to B3
    //
    B_Task_Ptr = &B3;
    //
}

void B3(void) //  SPARE
{
    TTPLPFC_HAL_toggleLED();

#if TTPLPFC_LAB == 4
    TTPLPFC_startLLC();

    TTPLPFC_llcStatusFlag();
#endif
    //
    //the next time CpuTimer1 'counter' reaches Period value go to B1
    //
    B_Task_Ptr = &B1;
}

//
// No more.
//
